@@ -892,8 +892,13 @@ static void display_fortune_dialog(FishApplet* fish)
 		gtk_label_set_ellipsize (GTK_LABEL (fish->fortune_label),
 					 PANGO_ELLIPSIZE_MIDDLE);
 		fish->fortune_cmd_label = gtk_label_new ("");
+#if GTK_CHECK_VERSION (3, 16, 0)
+		gtk_label_set_xalign (GTK_LABEL (fish->fortune_cmd_label), 0.0);
+		gtk_label_set_yalign (GTK_LABEL (fish->fortune_cmd_label), 0.5);
+#else
 		gtk_misc_set_alignment (GTK_MISC (fish->fortune_cmd_label),
 					0, 0.5);
+#endif
 
 		vbox = gtk_dialog_get_content_area (GTK_DIALOG (fish->fortune_dialog));
 		gtk_box_pack_start (GTK_BOX (vbox),
@@ -1476,9 +1481,11 @@ static gboolean fish_applet_draw(GtkWidget* widget, cairo_t *cr, FishApplet* fis
 static gboolean fish_applet_expose_event(GtkWidget* widget, GdkEventExpose* event, FishApplet* fish)
 #endif
 {
+#if !GTK_CHECK_VERSION (3, 0, 0)
 	GdkWindow    *window;
 	GtkStyle     *style;
 	GtkStateType  state;
+#endif
 	int width, height;
 	int src_x, src_y;
 
@@ -1490,9 +1497,11 @@ static gboolean fish_applet_expose_event(GtkWidget* widget, GdkEventExpose* even
 
 	g_assert (fish->n_frames > 0);
 
+#if !GTK_CHECK_VERSION (3, 0, 0)
 	window = gtk_widget_get_window (widget);
 	style = gtk_widget_get_style (widget);
 	state = gtk_widget_get_state (widget);
+#endif
 
 #if GTK_CHECK_VERSION(3, 0, 0)
 	width = cairo_xlib_surface_get_width (fish->surface);
