@@ -76,7 +76,12 @@ display_popup_window (GdkScreen *screen)
 
 	image = gtk_image_new_from_icon_name (PANEL_ICON_FORCE_QUIT,
 					      GTK_ICON_SIZE_DIALOG);
+#if GTK_CHECK_VERSION (3, 0, 0)
+	gtk_widget_set_halign (image, GTK_ALIGN_CENTER);
+	gtk_widget_set_valign (image, GTK_ALIGN_CENTER);
+#else
 	gtk_misc_set_alignment (GTK_MISC (image), 0.5, 0.5);
+#endif
 	gtk_box_pack_start (GTK_BOX (vbox), image, TRUE, TRUE, 4);
 	gtk_widget_show (image);
 
@@ -345,8 +350,8 @@ panel_force_quit (GdkScreen *screen,
 	root = gdk_screen_get_root_window (screen);
 
 	gdk_window_add_filter (root, (GdkFilterFunc) popup_filter, popup);
-
-	cross = gdk_cursor_new (GDK_CROSS);
+	cross = gdk_cursor_new_for_display (gdk_display_get_default (),
+	                                    GDK_CROSS);
 #if GTK_CHECK_VERSION (3, 0, 0)
 	display = gdk_window_get_display (root);
 	device_manager = gdk_display_get_device_manager (display);
