@@ -24,7 +24,7 @@
 #include "panel-util.h"
 #include "panel-marshal.h"
 #include "panel-typebuiltins.h"
-#include "mate-panel-applet-frame.h"
+#include "panel-applet-frame.h"
 #include "panel-globals.h"
 #include "panel-profile.h"
 #include "panel-lockdown.h"
@@ -80,21 +80,27 @@ static void panel_widget_cremove        (GtkContainer     *container,
 					 GtkWidget        *widget);
 static void panel_widget_dispose        (GObject *obj);
 static void panel_widget_finalize       (GObject          *obj);
+
+#if GTK_CHECK_VERSION (3, 18, 0)
+#elif GTK_CHECK_VERSION (3, 0, 0)
 static void panel_widget_realize        (GtkWidget        *widget);
 static void panel_widget_unrealize      (GtkWidget        *panel);
-#if GTK_CHECK_VERSION (3, 0, 0)
 static void panel_widget_state_flags_changed (GtkWidget *widget,
 					     GtkStateFlags previous_state);
 static void panel_widget_style_updated (GtkWidget *widget);
 #else
+static void panel_widget_realize        (GtkWidget        *widget);
+static void panel_widget_unrealize      (GtkWidget        *panel);
 static void panel_widget_state_changed  (GtkWidget        *widget,
 					 GtkStateType      previous_state);
 static void panel_widget_style_set      (GtkWidget        *widget,
 					 GtkStyle         *previous_style);
 #endif
 
+#if !GTK_CHECK_VERSION (3, 18, 0)
 static void panel_widget_background_changed (PanelBackground *background,
 					     PanelWidget     *panel);
+#endif
 
 static void panel_widget_push_move_applet   (PanelWidget      *panel,
                                              GtkDirectionType  dir);
@@ -438,13 +444,18 @@ panel_widget_class_init (PanelWidgetClass *class)
 	widget_class->size_request = panel_widget_size_request;
 #endif
 	widget_class->size_allocate = panel_widget_size_allocate;
+#if GTK_CHECK_VERSION (3, 19, 0)
+	gtk_widget_class_set_css_name (widget_class, "PanelWidget");
+#endif
+#if GTK_CHECK_VERSION (3, 18, 0)
+#elif GTK_CHECK_VERSION (3, 0, 0)
 	widget_class->realize = panel_widget_realize;
 	widget_class->unrealize = panel_widget_unrealize;
-	widget_class->focus = panel_widget_real_focus;
-#if GTK_CHECK_VERSION (3, 0, 0)
 	widget_class->state_flags_changed = panel_widget_state_flags_changed;
 	widget_class->style_updated = panel_widget_style_updated;
 #else
+	widget_class->realize = panel_widget_realize;
+	widget_class->unrealize = panel_widget_unrealize;
 	widget_class->state_changed = panel_widget_state_changed;
 	widget_class->style_set = panel_widget_style_set;
 #endif
@@ -1361,7 +1372,7 @@ queue_resize_on_all_applets(PanelWidget *panel)
 		gtk_widget_queue_resize (ad->applet);
 	}
 }
-
+#if !GTK_CHECK_VERSION(3, 18, 0)
 static void
 panel_widget_set_background_region (PanelWidget *panel)
 {
@@ -1384,7 +1395,7 @@ panel_widget_set_background_region (PanelWidget *panel)
 		allocation.width,
 		allocation.height);
 }
-
+#endif
 static void
 panel_widget_size_allocate(GtkWidget *widget, GtkAllocation *allocation)
 {
@@ -1587,12 +1598,10 @@ panel_widget_size_allocate(GtkWidget *widget, GtkAllocation *allocation)
 			gtk_widget_size_allocate(ad->applet,&challoc);
 		}
 	}
-	if(panel->orient == GTK_ORIENTATION_HORIZONTAL)
-		panel->thick = allocation->height;
-	else
-		panel->thick = allocation->width;
 
+#if !GTK_CHECK_VERSION(3, 18, 0)
 	panel_widget_set_background_region (panel);
+#endif
 }
 
 gboolean
@@ -1634,86 +1643,88 @@ panel_widget_is_cursor(PanelWidget *panel, int overlap)
 	return FALSE;
 }
 
+#if GTK_CHECK_VERSION (3, 18, 0)
+#elif GTK_CHECK_VERSION (3, 0, 0)
 static void
-#if GTK_CHECK_VERSION (3, 0, 0)
 panel_widget_set_background_default_style (GtkWidget *widget)
-#else
-panel_widget_style_set (GtkWidget *widget, GtkStyle  *previous_style)
-#endif
 {
-#if GTK_CHECK_VERSION (3, 0, 0)
 	GtkStyleContext *context;
 	GtkStateFlags state;
-	GdkRGBA bg_color;
+	GdkRGBA *bg_color;
 	cairo_pattern_t *bg_image;
-#else
-	GtkStyle     *style;
-	GtkStateType  state;
-#endif
+	PanelBackground *background;
 
 	if (gtk_widget_get_realized (widget)) {
-#if GTK_CHECK_VERSION (3, 0, 0)
 		context = gtk_widget_get_style_context (widget);
 		state = gtk_widget_get_state_flags (widget);
+		background = &PANEL_WIDGET (widget)->background;
+		panel_background_apply_css (&PANEL_WIDGET (widget)->background, widget);
 		gtk_style_context_add_class(context,"gnome-panel-menu-bar");
-		gtk_style_context_add_class(context,"mate-panel-menu-bar");
-		panel_background_apply_css (widget, &PANEL_WIDGET (widget)->background);
+		gtk_style_context_add_class(context,"mate-panel-menu-bar");		
 
-		gtk_style_context_get_background_color (context, state, &bg_color);
-		gtk_style_context_get (context, state, "background-image", &bg_image, NULL);
-#else
-		style = gtk_widget_get_style (widget);
-		state = gtk_widget_get_state (widget);
-#endif
+		gtk_style_context_get (context, state,
+		                       "background-color", &bg_color,
+		                       "background-image", &bg_image,
+		                       NULL);
 
-		panel_background_set_default_style (
-			&PANEL_WIDGET (widget)->background,
-#if GTK_CHECK_VERSION (3, 0, 0)
-			&bg_color,
-			bg_image);
+		panel_background_set_default_style (background,
+		                                    bg_color,
+		                                    bg_image);
 
-			if (bg_image)
-				cairo_pattern_destroy (bg_image);
-#else
-			&style->bg [state],
-			style->bg_pixmap [state]);
-#endif
+		if (bg_image)
+			cairo_pattern_destroy (bg_image);
 	}
 }
 
-static void
-#if GTK_CHECK_VERSION (3, 0, 0)
-panel_widget_style_updated (GtkWidget *widget)
-#else
-panel_widget_state_changed (GtkWidget    *widget,
-			    GtkStateType  previous_state)
-#endif
-{
-#if GTK_CHECK_VERSION (3, 0, 0)
-	GTK_WIDGET_CLASS (panel_widget_parent_class)->style_updated (widget);
-	panel_widget_set_background_default_style (widget);
-#else
-	GtkStyle     *style;
-	GtkStateType  state;
-
-	if (gtk_widget_get_realized (widget)) {
-		style = gtk_widget_get_style (widget);
-		state = gtk_widget_get_state (widget);
-
-		panel_background_set_default_style (
-			&PANEL_WIDGET (widget)->background,
-			&style->bg [state],
-			style->bg_pixmap [state]);
-	}
-#endif
-}
-
-#if GTK_CHECK_VERSION (3, 0, 0)
 static void
 panel_widget_state_flags_changed (GtkWidget *widget, GtkStateFlags previous_state)
 {
 	panel_widget_set_background_default_style (widget);
 }
+
+static void
+panel_widget_style_updated (GtkWidget *widget)
+{
+	GTK_WIDGET_CLASS (panel_widget_parent_class)->style_updated (widget);
+	panel_widget_set_background_default_style (widget);
+}
+
+#else
+
+static void
+panel_widget_state_changed (GtkWidget    *widget,
+			    GtkStateType  previous_state)
+{
+	GtkStyle     *style;
+	GtkStateType  state;
+
+	if (gtk_widget_get_realized (widget)) {
+		style = gtk_widget_get_style (widget);
+		state = gtk_widget_get_state (widget);
+
+		panel_background_set_default_style (
+			&PANEL_WIDGET (widget)->background,
+			&style->bg [state],
+			style->bg_pixmap [state]);
+	}
+}
+
+static void
+panel_widget_style_set (GtkWidget *widget, GtkStyle  *previous_style)
+{
+	GtkStyle     *style;
+	GtkStateType  state;
+
+	if (gtk_widget_get_realized (widget)) {
+		style = gtk_widget_get_style (widget);
+		state = gtk_widget_get_state (widget);
+
+		panel_background_set_default_style (
+			&PANEL_WIDGET (widget)->background,
+			&style->bg [state],
+			style->bg_pixmap [state]);
+	}
+}
 #endif
 
 static gboolean
@@ -1721,11 +1732,13 @@ toplevel_configure_event (GtkWidget         *widget,
 			  GdkEventConfigure *event,
 			  PanelWidget       *panel)
 {
+#if !GTK_CHECK_VERSION(3, 18, 0)
 	panel_widget_set_background_region (panel);
-
+#endif
 	return FALSE;
 }
 
+#if !GTK_CHECK_VERSION(3, 18, 0)
 static void
 panel_widget_realize (GtkWidget *widget)
 {
@@ -1748,9 +1761,9 @@ panel_widget_realize (GtkWidget *widget)
 
 	/* For auto-hidden panels with a colored background, we need native
 	 * windows to avoid some uglyness on unhide */
-	gdk_window_ensure_native (window);
 #endif
 
+
 #if GTK_CHECK_VERSION (3, 0, 0)
 	panel_widget_set_background_default_style (widget);
 #else
@@ -1762,7 +1775,7 @@ panel_widget_realize (GtkWidget *widget)
 
 	panel_background_realized (&panel->background, window);
 }
-
+#endif
 static void
 panel_widget_unrealize (GtkWidget *widget)
 {
@@ -1786,9 +1799,9 @@ panel_widget_finalize (GObject *obj)
 	g_return_if_fail (PANEL_IS_WIDGET (obj));
 
 	panel = PANEL_WIDGET (obj);
-
+#if !GTK_CHECK_VERSION(3, 18, 0)
 	panel_background_free (&panel->background);
-
+#endif
 	if (panel->applets_hints != NULL)
 		g_free (panel->applets_hints);
 	panel->applets_hints = NULL;
@@ -1863,7 +1876,6 @@ panel_widget_init (PanelWidget *panel)
 	
 	panel->packed        = FALSE;
 	panel->orient        = GTK_ORIENTATION_HORIZONTAL;
-	panel->thick         = PANEL_MINIMUM_WIDTH;
 	panel->size          = G_MAXINT;
 	panel->applet_list   = NULL;
 	panel->master_widget = NULL;
@@ -1873,10 +1885,11 @@ panel_widget_init (PanelWidget *panel)
 	panel->nb_applets_size_hints = 0;
 	panel->applets_hints = NULL;
 	panel->applets_using_hint = NULL;
-
+#if !GTK_CHECK_VERSION(3, 18, 0)
 	panel_background_init (&panel->background,
 			       (PanelBackgroundChangedNotify) panel_widget_background_changed,
 			       panel);
+#endif
 
 	panels = g_slist_append (panels, panel);
 }
@@ -2014,9 +2027,13 @@ panel_widget_applet_drag_start (PanelWidget *panel,
 		GdkDisplay *display;
 		GdkDevice *pointer;
 		GdkDeviceManager *device_manager;
+#endif
 
+		fleur_cursor = gdk_cursor_new_for_display (gdk_display_get_default (),
+		                                           GDK_FLEUR);
+
+#if GTK_CHECK_VERSION (3, 0, 0)
 		display = gdk_window_get_display (window);
-		fleur_cursor = gdk_cursor_new_for_display (display, GDK_FLEUR);
 		device_manager = gdk_display_get_device_manager (display);
 		pointer = gdk_device_manager_get_client_pointer (device_manager);
 		status = gdk_device_grab (pointer, window,
@@ -2026,8 +2043,6 @@ panel_widget_applet_drag_start (PanelWidget *panel,
 
 		g_object_unref (fleur_cursor);
 #else
-		fleur_cursor = gdk_cursor_new (GDK_FLEUR);
-
 		status = gdk_pointer_grab (window, FALSE,
 					   APPLET_EVENT_MASK, NULL,
 					   fleur_cursor, time_);
@@ -2515,7 +2530,11 @@ panel_widget_applet_key_press_event (GtkWidget   *widget,
 	if (!mate_panel_applet_in_drag)
 		return FALSE;
 
+#if GTK_CHECK_VERSION (3, 0, 0)
 	return gtk_bindings_activate (G_OBJECT (panel),
+#else
+	return gtk_bindings_activate (GTK_OBJECT (panel),
+#endif
 				      ((GdkEventKey *)event)->keyval, 
 				      ((GdkEventKey *)event)->state);	
 }
@@ -2741,7 +2760,6 @@ panel_widget_add (PanelWidget *panel,
 		ad->pos = pos;
 		ad->constrained = pos;
 		ad->drag_off = 0;
-		ad->no_die = 0;
 		ad->size_constrained = FALSE;
 		ad->expand_major = FALSE;
 		ad->expand_minor = FALSE;
@@ -2812,8 +2830,6 @@ panel_widget_reparent (PanelWidget *old_panel,
 	gtk_widget_queue_resize (GTK_WIDGET (new_panel));
 	gtk_widget_queue_resize (GTK_WIDGET (old_panel));
 
-	ad->no_die++;
-
 	panel_widget_reset_saved_focus (old_panel);
 	if (gtk_container_get_focus_child (GTK_CONTAINER (old_panel)) == applet)
 		focus_widget = gtk_window_get_focus (GTK_WINDOW (old_panel->toplevel));
@@ -2837,8 +2853,6 @@ panel_widget_reparent (PanelWidget *old_panel,
 
 	gdk_flush();
 
-	ad->no_die--;
-
 	emit_applet_moved (new_panel, ad);
 
 	return TRUE;
@@ -2880,6 +2894,14 @@ panel_widget_set_size (PanelWidget *panel_widget,
 	gtk_widget_queue_resize (GTK_WIDGET (panel_widget));
 }
 
+#if GTK_CHECK_VERSION (3, 18, 0)
+void
+panel_widget_emit_background_changed (PanelWidget *panel)
+{
+	g_signal_emit (panel, panel_widget_signals [BACK_CHANGE_SIGNAL], 0);
+}
+
+#else
 static void
 panel_widget_background_changed (PanelBackground *background,
 				 PanelWidget     *panel)
@@ -2890,7 +2912,7 @@ panel_widget_background_changed (PanelBackground *background,
 		       panel_widget_signals [BACK_CHANGE_SIGNAL],
 		       0);
 }
-
+#endif
 static void 
 panel_widget_push_move_applet (PanelWidget     *panel,
                                GtkDirectionType dir)
